@@ -16,6 +16,7 @@
 
 use crate::decimal::Decimal;
 use std::convert::TryFrom;
+use std::iter::{Product, Sum};
 use std::ops::{
     Add, AddAssign, Div, DivAssign, Mul, MulAssign, Neg, Rem, RemAssign, Sub, SubAssign,
 };
@@ -145,6 +146,20 @@ impl RemAssign for Decimal {
     }
 }
 
+impl Sum for Decimal {
+    #[inline(always)]
+    fn sum<I: Iterator<Item = Self>>(iter: I) -> Self {
+        iter.fold(Decimal::ZERO, Add::add)
+    }
+}
+
+impl Product for Decimal {
+    #[inline(always)]
+    fn product<I: Iterator<Item = Self>>(iter: I) -> Self {
+        iter.fold(Decimal::ONE, Mul::mul)
+    }
+}
+
 macro_rules! impl_arith_with_num {
     ($op: ident { $method: ident } $int: ty) => {
         impl $op<$int> for Decimal {
@@ -708,4 +723,33 @@ mod tests {
         );
         assert_rem("3", "5e42", "3");
     }
+
+    #[test]
+    fn test_sum() {
+        fn assert_sum(vals: &[&str], expected: &str) {
+            let result: Decimal = vals.iter().map(|val| val.parse::<Decimal>().unwrap()).sum();
+            let expected = expected.parse::<Decimal>().unwrap();
+            assert_eq!(result, expected);
+        }
+
+        assert_sum(&["1", "10", "100", "1000", "10000"], "11111");
+        assert_sum(&["-1", "-10", "-100", "-1000", "-10000"], "-11111");
+        assert_sum(&["0", "0", "0", "0", "0"], "0");
+    }
+
+    #[test]
+    fn test_product() {
+        fn assert_product(vals: &[&str], expected: &str) {
+            let result: Decimal = vals
+                .iter()
+                .map(|val| val.parse::<Decimal>().unwrap())
+                .product();
+            let expected = expected.parse::<Decimal>().unwrap();
+            assert_eq!(result, expected);
+        }
+
+        assert_product(&["1", "2", "3", "4", "5"], "120");
+        assert_product(&["-1", "-2", "-3", "-4", "-5"], "-120");
+        assert_product(&["0", "0", "0", "0", "0"], "0");
+    }
 }
