@@ -189,8 +189,9 @@ fn parse_str(s: &[u8]) -> Result<(Decimal, &[u8]), DecimalParseError> {
         s,
     ) = parse_decimal(s)?;
 
-    let precision = if integral == &b"0"[..] {
-        fractional.len() as u32
+    let precision = if integral == b"0" {
+        let zero_count = fractional.iter().take_while(|i| **i == b'0').count();
+        (fractional.len() - zero_count) as u32
     } else {
         (integral.len() + fractional.len()) as u32
     };
@@ -306,6 +307,7 @@ mod tests {
         assert_parse_overflow("1e1000");
         assert_parse_overflow("1e127");
         assert_parse_overflow("1e-131");
+        assert_parse_overflow("999999999999999999999999999999999999990");
     }
 
     fn assert_parse<S: AsRef<str>, V: AsRef<str>>(s: S, expected: V) {
@@ -333,6 +335,10 @@ mod tests {
             "99999999999999999999999999999999999999",
             "99999999999999999999999999999999999999",
         );
+        assert_parse(
+            "0099999999999999999999999999999999999999",
+            "99999999999999999999999999999999999999",
+        );
         assert_parse(
             "-99999999999999999999999999999999999999",
             "-99999999999999999999999999999999999999",
@@ -364,6 +370,11 @@ mod tests {
         );
         assert_parse("000000000123.000000000123", "123.000000000123");
         assert_parse("-000000000123.000000000123", "-123.000000000123");
+        assert_parse(
+            "00.000000000000000000000000000000000000123",
+            "0.000000000000000000000000000000000000123",
+        );
+        assert_parse("00.000000000000000000000000000000000000123e-87", "0.000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000123");
 
         // Scientific notation
         assert_parse("0e0", "0");
